# SPDX-FileCopyrightText: Daniele Nicolodi <daniele@grinta.net>
# SPDX-License-Identifier: Apache-2.0 OR LGPL-2.1-or-later

import datetime
import os

try:
    import tomllib
except ImportError:
    import tomli as tomllib

path = os.path.join(os.path.dirname(__file__), '..', 'pyproject.toml')
with open(path, 'rb') as fd:
    pyproject = tomllib.load(fd)['project']

project = pyproject['name']
version = pyproject['version']
copyright = f'2021\N{EN DASH}{datetime.date.today().year} Daniele Nicolodi'
author = 'Daniele Nicolodi'
language = 'en'
html_theme = 'furo'
html_title = f'{project} {version}'
extensions = [
    'sphinx.ext.intersphinx',
    'sphinx.ext.githubpages',
]
intersphinx_mapping = {
    'python': ('https://docs.python.org/3', None),
}



# SPDX-FileCopyrightText: Daniele Nicolodi <daniele@grinta.net>
# SPDX-License-Identifier: Apache-2.0 OR LGPL-2.1-or-later

import sys
from pathlib import Path
from Cython.Tempita import sub

template = Path(sys.argv[1]).read_text('utf8')
output = sub(template)
Path(sys.argv[2]).write_text(output, 'utf8')



# SPDX-FileCopyrightText: Daniele Nicolodi <daniele@grinta.net>
# SPDX-License-Identifier: Apache-2.0 OR LGPL-2.1-or-later

import os
import sys
import siphash24

# Python uses a randomized seed unless told otherwise
assert int(os.environ.get('PYTHONHASHSEED', '')) == 0

# Ensure that Python is built to use Sip Hash as internal hashing function.
# The FNV hashing function is used on platforms that require aligned memory
# access for integers. See PEP-456 for details
algorithm = sys.hash_info.algorithm
assert algorithm in {'siphash13', 'siphash24'}

# Python up to release 3.10 uses Sip Hash 13, Python 3.11 and later uses Sip Hash 13
siphash = getattr(siphash24, algorithm)

DATA = b'spam'

assert siphash(DATA).intdigest() == hash(DATA)
assert int.from_bytes(siphash(DATA).digest(), 'little', signed=True) == hash(DATA)



# SPDX-FileCopyrightText: Daniele Nicolodi <daniele@grinta.net>
# SPDX-License-Identifier: Apache-2.0 OR LGPL-2.1-or-later

import array
import binascii
import unittest
import siphash24


class TestSipHas24(unittest.TestCase):

    def test_siphash24(self):
        h = siphash24.siphash24()
        self.assertEqual(h.digest(), b'\xd7\x00ws\x9dK\x92\x1e')

    def test_siphash24_data(self):
        h = siphash24.siphash24(b'')
        self.assertEqual(h.digest(), b'\xd7\x00ws\x9dK\x92\x1e')

    def test_siphash24_key(self):
        h = siphash24.siphash24(key=b'')
        self.assertEqual(h.digest(), b'\xd7\x00ws\x9dK\x92\x1e')

    def test_siphash24_long_key(self):
        with self.assertRaises(ValueError):
            h = siphash24.siphash24(key=b'\x00' * 17)

    def test_siphash24_data_and_key(self):
        h = siphash24.siphash24(b'', key=b'')
        self.assertEqual(h.digest(), b'\xd7\x00ws\x9dK\x92\x1e')

    def test_siphash24_hexdigest(self):
        h = siphash24.siphash24()
        self.assertIsInstance(h.digest(), bytes)
        self.assertEqual(h.hexdigest(), binascii.hexlify(h.digest()).decode('ascii'))

    def test_siphash24_name(self):
        h = siphash24.siphash24()
        self.assertIsInstance(h.name, str)
        self.assertEqual(h.name, 'siphash24')

    def test_siphash24_digest_size(self):
        h = siphash24.siphash24()
        self.assertEqual(len(h.digest()), h.digest_size)
        self.assertEqual(len(h.hexdigest()), 2 * h.digest_size)

    def test_siphash24_reference(self):
        key = b'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f'
        data = b'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e'
        self.assertEqual(siphash24.siphash24(data, key=key).digest(), b'\xe5\x45\xbe\x49\x61\xca\x29\xa1')

    def test_siphahs24_reference_chunked(self):
        key = b'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f'
        data = b'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e'
        for i in range(len(data)):
            for j in range(i, len(data)):
                h = siphash24.siphash24(key=key)
                h.update(data[ :i])
                h.update(data[i:j])
                h.update(data[j: ])
                self.assertEqual(data[:i] + data[i:j] + data[j:], data)
                self.assertEqual(int.from_bytes(h.digest(), 'little'), 0xa129ca6149be45e5)

    def test_siphash24_update(self):
        key = b'\x22\x24\x41\x22\x55\x77\x88\x07\x23\x09\x23\x14\x0c\x33\x0e\x0f'
        data = b'\x01\x02\x03\x04\x05\x06\x07\x08\x09\x10\x11\x12\x13\x14\x15\x16'
        for i in range(len(data)):
            h = siphash24.siphash24(key=key)
            h.update(data[:i])
            h.update(data[i:])
            self.assertEqual(h.digest(), siphash24.siphash24(data, key=key).digest())

    def test_siphash24_types(self):
        with self.assertRaises(TypeError):
            h = siphash24.siphash24('spam')
        with self.assertRaises(TypeError):
            h = siphash24.siphash24(key='spam')
        with self.assertRaises(TypeError):
            h = siphash24.siphash24(1)
        with self.assertRaises(TypeError):
            h = siphash24.siphash24(key=1)

    def test_siphash24_buffer_interface(self):
        data = array.array('i', range(32))
        self.assertEqual(siphash24.siphash24(data).digest(), siphash24.siphash24(data.tobytes()).digest())


if __name__ == '__main__':
    unittest.main()
